## begin license ##
#
#    Slowfoot is an integration system for web-based applications.
#    Copyright (C) 2005-2009 Seek You Too B.V. (CQ2) http://www.cq2.nl
#
#    This file is part of Slowfoot.
#
#    Slowfoot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Slowfoot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Slowfoot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
from urllib2 import urlopen
from cq2utils_old import binderytools
from lrucache import LRUCache
from threading import Lock
from os import stat
import time

def getKeyFor(url):
	return hasattr(url, 'geturl') and url.geturl() \
			or hasattr(url, 'name') and url.name \
			or hasattr(url, 'uuid') and url.uuid \
			or	str(url)
	
def getDateFor(url):
	return hasattr(url, 'info') and url.info().getdate('last-modified') \
			or hasattr(url, 'name') and stat(url.name).st_mtime \
			or	time.time()

class AmaraCache:
	def __init__(self):
		self.cache = LRUCache(128)
		self.lock = Lock()

	def locked(self, method, *args):
		self.lock.acquire()
		try:
			return method(*args)
		finally:
			self.lock.release()

	def getFromCache(self, key):
		if key in self.cache:
			return self.cache[key]
		return None, None
			
	def setToCache(self, key, value):
		self.cache[key] = value

	def get(self, url):
		key = getKeyFor(url)
		newdate = getDateFor(url)
		
		amara, olddate = self.locked(self.getFromCache, key)
		if newdate == olddate:
			return amara
		amara = binderytools.bind_stream(url)
		self.locked(self.setToCache, key, (amara, newdate))
		
		return amara
