## begin license ##
#
#    Slowfoot is an integration system for web-based applications.
#    Copyright (C) 2005-2009 Seek You Too B.V. (CQ2) http://www.cq2.nl
#
#    This file is part of Slowfoot.
#
#    Slowfoot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Slowfoot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Slowfoot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

import magic

class MagicStream:
	def __init__(self, stream):
		self.stream = stream
		line = self.stream.readline()
		self.mimetype = magic.whatis(line)
		self.xml = 'xml' in self.mimetype
		self.line = self.convertToUniversalNewline(line)

	def __iter__(self):
		return self

	def close(self):
		self.stream.close()

	def convertToUniversalNewline(self, string):
		if 'text' in self.mimetype:
			return string.replace('\r\r','\r').replace('\r\n', '\n').replace('\r','\n')
		return string

	def next(self):
		if self.line:
			result = self.line
			self.line = None
		else:
			result = self.stream.next()
		return self.convertToUniversalNewline(result)

	def read(self, size = -1):
		if size > len(self.line) or size == -1:
			next = self.stream.read(size)
			if next:
				self.line += self.convertToUniversalNewline(next)
				return self.read(size)
		result = size == -1 and self.line or self.line[:size]
		self.line = self.line[max(size,0):]
		return result
